@@ -161,6 +161,12 @@ class Constant(_StrCaching, _StringReferenceCaching, _ConstOpMixin, Value):
         tys = [el.type for el in elems]
         return cls(types.LiteralStructType(tys), elems)
 
+    @property
+    def addrspace(self):
+        if not isinstance(self.type, types.PointerType):
+            raise TypeError("Only pointer constant have address spaces")
+        return self.type.addrspace
+
     def __eq__(self, other):
         if isinstance(other, Constant):
             return str(self) == str(other)
@@ -666,6 +672,48 @@ class ArgumentAttributes(AttributeSet):
                         'nocapture', 'nonnull', 'returned', 'signext',
                         'sret', 'zeroext'])
 
+    def __init__(self):
+        self._align = 0
+        self._dereferenceable = 0
+        self._dereferenceable_or_null = 0
+
+    @property
+    def align(self):
+        return self._align
+
+    @align.setter
+    def align(self, val):
+        assert isinstance(val, six.integer_types) and val >= 0
+        self._align = val
+
+    @property
+    def dereferenceable(self):
+        return self._dereferenceable
+
+    @dereferenceable.setter
+    def dereferenceable(self, val):
+        assert isinstance(val, six.integer_types) and val >= 0
+        self._dereferenceable = val
+
+    @property
+    def dereferenceable_or_null(self):
+        return self._dereferenceable_or_null
+
+    @dereferenceable_or_null.setter
+    def dereferenceable_or_null(self, val):
+        assert isinstance(val, six.integer_types) and val >= 0
+        self._dereferenceable_or_null = val
+
+    def _to_list(self):
+        attrs = sorted(self)
+        if self.align:
+            attrs.append('align {0:d}'.format(self.align))
+        if self.dereferenceable:
+            attrs.append('dereferenceable({0:d})'.format(self.dereferenceable))
+        if self.dereferenceable_or_null:
+            attrs.append('dereferenceable_or_null({0:d})'.format(self.dereferenceable_or_null))
+        return attrs
+
 
 class _BaseArgument(NamedValue):
     def __init__(self, parent, typ, name=''):
@@ -687,8 +735,9 @@ class Argument(_BaseArgument):
     """
 
     def __str__(self):
-        if self.attributes:
-            return "{0} {1} {2}".format(self.type, ' '.join(self.attributes),
+        attrs = self.attributes._to_list()
+        if attrs:
+            return "{0} {1} {2}".format(self.type, ' '.join(attrs),
                                         self.get_reference())
         else:
             return "{0} {1}".format(self.type, self.get_reference())
@@ -700,8 +749,9 @@ class ReturnValue(_BaseArgument):
     """
 
     def __str__(self):
-        if self.attributes:
-            return "{0} {1}".format(' '.join(self.attributes), self.type)
+        attrs = self.attributes._to_list()
+        if attrs:
+            return "{0} {1}".format(' '.join(attrs), self.type)
         else:
             return str(self.type)
 
